package com.pointrlabs.sample.map

import android.content.Context
import androidx.appcompat.app.AppCompatActivity
import com.pointrlabs.core.map.models.events_listeners.MapEventsListener
import com.pointrlabs.core.management.models.Building
import android.os.Bundle
import com.pointrlabs.sample.R
import com.pointrlabs.core.map.viewmodels.PTRMapWidgetConfiguration
import com.pointrlabs.core.map.views.PTRMapWidgetFragment
import com.pointrlabs.core.management.Pointr
import android.content.Intent
import com.pointrlabs.core.management.models.Site
import com.pointrlabs.core.map.views.PTRMapFragment

class MapActivity : AppCompatActivity(), MapEventsListener {

    private lateinit var building: Building

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_map_widget)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        val pointr = Pointr.getPointr() ?: return
        val siteManager = pointr.siteManager ?: return

        val buildingInternalIdentifier = intent.getIntExtra(BUNDLE_BUILDING_ID, 0)
        // if you want to delegate activity recreation handling to PointrSDK
        // get the widget instance after calling show method
        val configuration = PTRMapWidgetConfiguration.defaultConfiguration()
        val widgetFragment = PTRMapWidgetFragment.newInstance(configuration)
            .show(supportFragmentManager, R.id.fragment_container)

        building = siteManager.getBuilding(buildingInternalIdentifier) ?: run {
            val sites = siteManager.getSites()
            if (sites.isEmpty()) return
            val site = sites.last()
            val buildings = site.buildings
            if (buildings.isEmpty()) return
            buildings.last()
        }

        widgetFragment.loadAndSet(building)
        widgetFragment.addListener(this)
        this.title = building.title
        val permissionManager = pointr.permissionManager ?: return
        if (!permissionManager.hasLocationPermissionWhileInUse) {
            permissionManager.requestLocationPermissionForWhileInUse(this, 1)
        } else if (!permissionManager.hasLocationPermissionAlways) {
            permissionManager.requestLocationPermissionForAlways(this, 2)
        } else if (!permissionManager.hasBluetoothPermission) {
            permissionManager.requestBluetoothPermission(this, 3)
        } else if (!permissionManager.isLocationServicesEnabled) {
            permissionManager.requestLocationService(this)
        } else if (!permissionManager.isBluetoothServicesEnabled && permissionManager.hasBluetoothPermission) {
            permissionManager.requestBluetoothService()
        }
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        val permissionManager = Pointr.getPointr()?.permissionManager ?: return
        when (requestCode) {
            1, 421903 -> if (!permissionManager.hasLocationPermissionAlways) {
                permissionManager.requestLocationPermissionForAlways(this, 2)
            } else if (!permissionManager.hasBluetoothPermission) {
                permissionManager.requestBluetoothPermission(this, 3)
            } else if (!permissionManager.isLocationServicesEnabled) {
                permissionManager.requestLocationService(this)
            } else if (!permissionManager.isBluetoothServicesEnabled && permissionManager.hasBluetoothPermission) {
                permissionManager.requestBluetoothService()
            }
            2 -> if (!permissionManager.hasBluetoothPermission) {
                permissionManager.requestBluetoothPermission(this, 3)
            } else if (!permissionManager.isLocationServicesEnabled) {
                permissionManager.requestLocationService(this)
            } else if (!permissionManager.isBluetoothServicesEnabled && permissionManager.hasBluetoothPermission) {
                permissionManager.requestBluetoothService()
            }
            3, 421904 -> if (!permissionManager.isLocationServicesEnabled) {
                permissionManager.requestLocationService(this)
            } else if (!permissionManager.isBluetoothServicesEnabled && permissionManager.hasBluetoothPermission) {
                permissionManager.requestBluetoothService()
            }
            42191 -> if (!permissionManager.isBluetoothServicesEnabled && permissionManager.hasBluetoothPermission) {
                permissionManager.requestBluetoothService()
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        val permissionManager = Pointr.getPointr()?.permissionManager ?: return
        when (requestCode) {
            42191 -> if (!permissionManager.isBluetoothServicesEnabled && permissionManager.hasBluetoothPermission) {
                permissionManager.requestBluetoothService()
            }
        }
    }

    override fun mapDidDetectFocusedSite(
        mapFragment: PTRMapFragment,
        site: Site?,
        buildings: List<Building>
    ) {
        if (buildings.isEmpty()) {
            title = ""
            return
        }
        if (buildings.contains(building)) return
        building = buildings.first()
        title = building.title
    }


    companion object {
        private const val BUNDLE_BUILDING_ID = "BUNDLE_BUILDING_ID"

        /**
         * Creates an intent to start this activity
         *
         * @param venueId to load and display the [PTRMapWidgetFragment]
         * @return intent to start []
         */
        @JvmStatic
        fun newIntent(
            context: Context,
            venueId: Int
        ): Intent {
            val intent = Intent(context, MapActivity::class.java)
            intent.putExtra(BUNDLE_BUILDING_ID, venueId)
            return intent
        }
    }
}